package SOCC::File::Cmd;

our $VERSION = '0.03';

use strict;
use English;
use Carp;
use Fcntl;
use POSIX ('PIPE_BUF');
use fields ('_path', '_timeout', '_bufsiz', '_buf');



sub new {
	my ($class) = shift @_;
	my %args = (timeout => 5, @_);
	
	my $self = fields::new($class);
	$self->{_path} = $args{path};
	$self->{_timeout} = $args{timeout};
	$self->{_bufsiz} = $args{bufsiz};
	$self->{_buf} = '';
	
	return $self;
}



sub write {
	my ($self, $arg) = @_;
	
	# Overloading write() method.
	if (ref $arg && $arg->isa('SOCC::ServiceResult')) {
		$arg = _service_result_to_externcmd($arg);
	}
	# Otherwise must be a string.
	
	if (defined $self->{_bufsiz}) {
		# Truncate arg to make sure that write will be atomic.  Do not
		# remove the newline if there was one at the end of the message.
		if (length($arg) > $self->{_bufsiz}) {
			if ($arg =~ m/([\r\n])$/) {
				$arg = substr($arg, 0, $self->{_bufsiz} - 1) . $1;
			} else {
				$arg = substr($arg, 0, $self->{_bufsiz});
			}
		}
		# Flush out buffered data if we need room.
		if (length($self->{_buf})  + length($arg)  > $self->{_bufsiz}) {
			$self->flush;
		}
		# Buffer data.
		$self->{_buf} .= $arg;
	} else {
		# No buffering.
		$self->flush($arg);
	}
}



sub _service_result_to_externcmd {
	my ($event) = @_;
	
	my $output = sprintf(
		"[ \%d ] PROCESS_SERVICE_CHECK_RESULT;\%s;\%s;\%d;\%s\n",
		$event->time_t(), $event->host_name(), $event->service_descr(),
		$event->return_code(), $event->output());
	return $output;
}



sub flush {
	my ($self, $arg) = @_;
	
	$arg .= $self->{_buf};
	if (length $arg > 0) {
		eval {
			# Start timer.
			local $SIG{ALRM} = sub {
				croak("Timed out writing to '", $self->{_path},	"', stopped ")
			};
			alarm($self->{_timeout});
			
			# Nagios has the nasty habit of not closing the fifo properly
			# when shutting down.  To make sure we are not talking to an 
			# unlinked fifo we need to open and close for each write.
			sysopen(FH, $self->{_path}, O_WRONLY | O_APPEND) or
				croak("Could not open '", $self->{_path}, "', stopped ");
			local $SIG{PIPE} = 'IGNORE';
			syswrite(FH, $arg) == length $arg or
				croak("Could not write to '", $self->{_path}, "', stopped ");
			close(FH);
			
			$self->{_buf} = '';
		};
		
		# Turn off timer.
		alarm(0);
		
		die($EVAL_ERROR) unless $EVAL_ERROR eq '';
	}
}



sub DESTROY {
	my $self = $_[0];
	
	$self->flush();
}



1;
