#!/usr/bin/perl


use Test::More tests => 12;
use strict;
use warnings;
use English;
use POSIX ('PIPE_BUF');
use File::Copy;
use File::Path;



use SOCC::ServiceResult;
use_ok('SOCC::::File::Cmd');


rmtree('tmp');
mkdir('tmp');
{
	# Setting up a service event result for testing.
	my $service_result = SOCC::ServiceResult->new(time_t => '1234567',
		host_name => 'somehost.its',
		service_descr => 'Some Service Desc',
		return_code => 1,
		output => 'Service is at Warning levels.');	
	copy('/dev/null', 'tmp/test_cmdfile');
	open(FH, "< tmp/test_cmdfile");
	
	my $cmd_file = SOCC::File::Cmd->new(path => 'tmp/test_cmdfile',
		timeout => 2);
	
	# Normal operation.
	$cmd_file->write($service_result);
	pass;
	
	my $line = <FH>;
	ok(defined $line);
	
	# Simulate Nagios crash, oops.
	unlink('tmp/test_cmdfile');
	eval { $cmd_file->write($service_result) };
	ok($EVAL_ERROR =~ m/^Could not open '/);
	
	# Simulate timeout
	system('mkfifo tmp/test_cmdfile');
	eval {$cmd_file->write($service_result)};
	ok($EVAL_ERROR =~ m/^Timed out writing to '/);
	unlink 'tmp/test_cmdfile';
	
	# Simulate Nagios starting again.
	copy('/dev/null', 'tmp/test_cmdfile');
	open(FH, "< tmp/test_cmdfile");
	$cmd_file->write($service_result);
	pass;
	$line = <FH>;
	ok(defined $line);
	
	close(FH);
}


# Show that strings as well as ServiceResult objects can be written.
{
	my $string = "A RAW STRING\n";
	my $service_result = SOCC::ServiceResult->new(time_t => '1234567',
		host_name => 'somehost.its',
		service_descr => 'Some Service Desc',
		return_code => 1,
		output => 'Warning Service has problems.|device,val=1,val2=2');
	copy('/dev/null', 'tmp/test_cmdfile2');
	my $cmd_file = SOCC::File::Cmd->new(path => 'tmp/test_cmdfile2',
		timeout => 0);
	$cmd_file->write($string);
	open(WRITTEN, "< tmp/test_cmdfile2");
	my $written = <WRITTEN>;
	close(WRITTEN);
	ok($string eq $written);
}


{
	# Demonstrating new buffering functionality, buffering is turned on if
	# new(bufsiz=>) is supplied.  service_result data will be buffered until
	# a) flush() is called
	# b) write() is called and buffering any more data would take us over
	# bufsiz.
	# c) the object is destroyed.
	#
	# NOTE:
	# 1) Data is written out so that message is not
	# broken up across a syswrite.
	# 2) If any one message over bufsiz in length then it will be 
	# truncated to bufsiz.
	
	my @events = (
		"MessageA\n",
		"MessageB a longer message\n",
		'MessageC an overly large message, ' . ' junk' x 100 . "\n",
	);
	my $bufsiz = length $events[1];
	
	copy('/dev/null', 'tmp/test_cmdfile');
	open(FH, "< tmp/test_cmdfile");
	my $cmd_file = SOCC::File::Cmd->new(bufsiz => $bufsiz,
		path => 'tmp/test_cmdfile',
		timeout => 0);
	
	# Data is buffered.
	$cmd_file->write($events[0]);
	my $line = <FH>;
	ok (! defined $line);
	
	# Previous buffered data written out due to bufsiz.
	$cmd_file->write($events[1]);
	$line = <FH>;
	ok ($line eq $events[0]);
	
	# Buffered data written out due to flush().
	$cmd_file->flush();
	$line = <FH>;
	ok($line eq $events[1]);
	
	# Overly long data is truncated.
	$cmd_file->write($events[2]);
	$cmd_file->flush();
	$line = <FH>;
	ok(length($line) == $bufsiz);
	
	close FH;
}

